// Copyright 2023 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/resourcemanager/v3/tag_holds.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_RESOURCEMANAGER_V3_TAG_HOLDS_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_RESOURCEMANAGER_V3_TAG_HOLDS_CLIENT_H

#include "google/cloud/resourcemanager/v3/tag_holds_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/no_await_tag.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <google/longrunning/operations.grpc.pb.h>
#include <memory>
#include <string>

namespace google {
namespace cloud {
namespace resourcemanager_v3 {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// Allow users to create and manage TagHolds for TagValues. TagHolds represent
/// the use of a Tag Value that is not captured by TagBindings but
/// should still block TagValue deletion (such as a reference in a policy
/// condition). This service provides isolated failure domains by cloud location
/// so that TagHolds can be managed in the same location as their usage.
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class TagHoldsClient {
 public:
  explicit TagHoldsClient(std::shared_ptr<TagHoldsConnection> connection,
                          Options opts = {});
  ~TagHoldsClient();

  ///@{
  /// @name Copy and move support
  TagHoldsClient(TagHoldsClient const&) = default;
  TagHoldsClient& operator=(TagHoldsClient const&) = default;
  TagHoldsClient(TagHoldsClient&&) = default;
  TagHoldsClient& operator=(TagHoldsClient&&) = default;
  ///@}

  ///@{
  /// @name Equality
  friend bool operator==(TagHoldsClient const& a, TagHoldsClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(TagHoldsClient const& a, TagHoldsClient const& b) {
    return !(a == b);
  }
  ///@}

  // clang-format off
  ///
  /// Creates a TagHold. Returns ALREADY_EXISTS if a TagHold with the same
  /// resource and origin exists under the same TagValue.
  ///
  /// @param parent  Required. The resource name of the TagHold's parent TagValue. Must be of
  ///  the form: `tagValues/{tag-value-id}`.
  /// @param tag_hold  Required. The TagHold to be created.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return A [`future`] that becomes satisfied when the LRO
  ///     ([Long Running Operation]) completes or the polling policy in effect
  ///     for this call is exhausted. The future is satisfied with an error if
  ///     the LRO completes with an error or the polling policy is exhausted.
  ///     In this case the [`StatusOr`] returned by the future contains the
  ///     error. If the LRO completes successfully the value of the future
  ///     contains the LRO's result. For this RPC the result is a
  ///     [google.cloud.resourcemanager.v3.TagHold] proto message.
  ///     The C++ class representing this message is created by Protobuf, using
  ///     the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [Long Running Operation]: https://google.aip.dev/151
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.resourcemanager.v3.CreateTagHoldRequest]: @googleapis_reference_link{google/cloud/resourcemanager/v3/tag_holds.proto#L121}
  /// [google.cloud.resourcemanager.v3.TagHold]: @googleapis_reference_link{google/cloud/resourcemanager/v3/tag_holds.proto#L87}
  ///
  // clang-format on
  future<StatusOr<google::cloud::resourcemanager::v3::TagHold>> CreateTagHold(
      std::string const& parent,
      google::cloud::resourcemanager::v3::TagHold const& tag_hold,
      Options opts = {});

  // clang-format off
  ///
  /// @copybrief CreateTagHold
  ///
  /// Specifying the [`NoAwaitTag`] immediately returns the
  /// [`google::longrunning::Operation`] that corresponds to the Long Running
  /// Operation that has been started. No polling for operation status occurs.
  ///
  /// [`NoAwaitTag`]: @ref google::cloud::NoAwaitTag
  ///
  // clang-format on
  StatusOr<google::longrunning::Operation> CreateTagHold(
      NoAwaitTag, std::string const& parent,
      google::cloud::resourcemanager::v3::TagHold const& tag_hold,
      Options opts = {});

  // clang-format off
  ///
  /// Creates a TagHold. Returns ALREADY_EXISTS if a TagHold with the same
  /// resource and origin exists under the same TagValue.
  ///
  /// @param request Unary RPCs, such as the one wrapped by this
  ///     function, receive a single `request` proto message which includes all
  ///     the inputs for the RPC. In this case, the proto message is a
  ///     [google.cloud.resourcemanager.v3.CreateTagHoldRequest].
  ///     Proto messages are converted to C++ classes by Protobuf, using the
  ///     [Protobuf mapping rules].
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return A [`future`] that becomes satisfied when the LRO
  ///     ([Long Running Operation]) completes or the polling policy in effect
  ///     for this call is exhausted. The future is satisfied with an error if
  ///     the LRO completes with an error or the polling policy is exhausted.
  ///     In this case the [`StatusOr`] returned by the future contains the
  ///     error. If the LRO completes successfully the value of the future
  ///     contains the LRO's result. For this RPC the result is a
  ///     [google.cloud.resourcemanager.v3.TagHold] proto message.
  ///     The C++ class representing this message is created by Protobuf, using
  ///     the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [Long Running Operation]: https://google.aip.dev/151
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.resourcemanager.v3.CreateTagHoldRequest]: @googleapis_reference_link{google/cloud/resourcemanager/v3/tag_holds.proto#L121}
  /// [google.cloud.resourcemanager.v3.TagHold]: @googleapis_reference_link{google/cloud/resourcemanager/v3/tag_holds.proto#L87}
  ///
  // clang-format on
  future<StatusOr<google::cloud::resourcemanager::v3::TagHold>> CreateTagHold(
      google::cloud::resourcemanager::v3::CreateTagHoldRequest const& request,
      Options opts = {});

  // clang-format off
  ///
  /// @copybrief CreateTagHold
  ///
  /// Specifying the [`NoAwaitTag`] immediately returns the
  /// [`google::longrunning::Operation`] that corresponds to the Long Running
  /// Operation that has been started. No polling for operation status occurs.
  ///
  /// [`NoAwaitTag`]: @ref google::cloud::NoAwaitTag
  ///
  // clang-format on
  StatusOr<google::longrunning::Operation> CreateTagHold(
      NoAwaitTag,
      google::cloud::resourcemanager::v3::CreateTagHoldRequest const& request,
      Options opts = {});

  // clang-format off
  ///
  /// @copybrief CreateTagHold
  ///
  /// This method accepts a `google::longrunning::Operation` that corresponds
  /// to a previously started Long Running Operation (LRO) and polls the status
  /// of the LRO in the background.
  ///
  // clang-format on
  future<StatusOr<google::cloud::resourcemanager::v3::TagHold>> CreateTagHold(
      google::longrunning::Operation const& operation, Options opts = {});

  // clang-format off
  ///
  /// Deletes a TagHold.
  ///
  /// @param name  Required. The resource name of the TagHold to delete. Must be of the form:
  ///  `tagValues/{tag-value-id}/tagHolds/{tag-hold-id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return A [`future`] that becomes satisfied when the LRO
  ///     ([Long Running Operation]) completes or the polling policy in effect
  ///     for this call is exhausted. The future is satisfied with an error if
  ///     the LRO completes with an error or the polling policy is exhausted.
  ///     In this case the [`StatusOr`] returned by the future contains the
  ///     error. If the LRO completes successfully the value of the future
  ///     contains the LRO's result. For this RPC the result is a
  ///     [google.cloud.resourcemanager.v3.DeleteTagHoldMetadata] proto message.
  ///     The C++ class representing this message is created by Protobuf, using
  ///     the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [Long Running Operation]: https://google.aip.dev/151
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.resourcemanager.v3.DeleteTagHoldMetadata]: @googleapis_reference_link{google/cloud/resourcemanager/v3/tag_holds.proto#L163}
  /// [google.cloud.resourcemanager.v3.DeleteTagHoldRequest]: @googleapis_reference_link{google/cloud/resourcemanager/v3/tag_holds.proto#L145}
  ///
  // clang-format on
  future<StatusOr<google::cloud::resourcemanager::v3::DeleteTagHoldMetadata>>
  DeleteTagHold(std::string const& name, Options opts = {});

  // clang-format off
  ///
  /// @copybrief DeleteTagHold
  ///
  /// Specifying the [`NoAwaitTag`] immediately returns the
  /// [`google::longrunning::Operation`] that corresponds to the Long Running
  /// Operation that has been started. No polling for operation status occurs.
  ///
  /// [`NoAwaitTag`]: @ref google::cloud::NoAwaitTag
  ///
  // clang-format on
  StatusOr<google::longrunning::Operation> DeleteTagHold(
      NoAwaitTag, std::string const& name, Options opts = {});

  // clang-format off
  ///
  /// Deletes a TagHold.
  ///
  /// @param request Unary RPCs, such as the one wrapped by this
  ///     function, receive a single `request` proto message which includes all
  ///     the inputs for the RPC. In this case, the proto message is a
  ///     [google.cloud.resourcemanager.v3.DeleteTagHoldRequest].
  ///     Proto messages are converted to C++ classes by Protobuf, using the
  ///     [Protobuf mapping rules].
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return A [`future`] that becomes satisfied when the LRO
  ///     ([Long Running Operation]) completes or the polling policy in effect
  ///     for this call is exhausted. The future is satisfied with an error if
  ///     the LRO completes with an error or the polling policy is exhausted.
  ///     In this case the [`StatusOr`] returned by the future contains the
  ///     error. If the LRO completes successfully the value of the future
  ///     contains the LRO's result. For this RPC the result is a
  ///     [google.cloud.resourcemanager.v3.DeleteTagHoldMetadata] proto message.
  ///     The C++ class representing this message is created by Protobuf, using
  ///     the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [Long Running Operation]: https://google.aip.dev/151
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.resourcemanager.v3.DeleteTagHoldMetadata]: @googleapis_reference_link{google/cloud/resourcemanager/v3/tag_holds.proto#L163}
  /// [google.cloud.resourcemanager.v3.DeleteTagHoldRequest]: @googleapis_reference_link{google/cloud/resourcemanager/v3/tag_holds.proto#L145}
  ///
  // clang-format on
  future<StatusOr<google::cloud::resourcemanager::v3::DeleteTagHoldMetadata>>
  DeleteTagHold(
      google::cloud::resourcemanager::v3::DeleteTagHoldRequest const& request,
      Options opts = {});

  // clang-format off
  ///
  /// @copybrief DeleteTagHold
  ///
  /// Specifying the [`NoAwaitTag`] immediately returns the
  /// [`google::longrunning::Operation`] that corresponds to the Long Running
  /// Operation that has been started. No polling for operation status occurs.
  ///
  /// [`NoAwaitTag`]: @ref google::cloud::NoAwaitTag
  ///
  // clang-format on
  StatusOr<google::longrunning::Operation> DeleteTagHold(
      NoAwaitTag,
      google::cloud::resourcemanager::v3::DeleteTagHoldRequest const& request,
      Options opts = {});

  // clang-format off
  ///
  /// @copybrief DeleteTagHold
  ///
  /// This method accepts a `google::longrunning::Operation` that corresponds
  /// to a previously started Long Running Operation (LRO) and polls the status
  /// of the LRO in the background.
  ///
  // clang-format on
  future<StatusOr<google::cloud::resourcemanager::v3::DeleteTagHoldMetadata>>
  DeleteTagHold(google::longrunning::Operation const& operation,
                Options opts = {});

  // clang-format off
  ///
  /// Lists TagHolds under a TagValue.
  ///
  /// @param parent  Required. The resource name of the parent TagValue. Must be of the form:
  ///  `tagValues/{tag-value-id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return a [StreamRange](@ref google::cloud::StreamRange)
  ///     to iterate of the results. See the documentation of this type for
  ///     details. In brief, this class has `begin()` and `end()` member
  ///     functions returning a iterator class meeting the
  ///     [input iterator requirements]. The value type for this iterator is a
  ///     [`StatusOr`] as the iteration may fail even after some values are
  ///     retrieved successfully, for example, if there is a network disconnect.
  ///     An empty set of results does not indicate an error, it indicates
  ///     that there are no resources meeting the request criteria.
  ///     On a successful iteration the `StatusOr<T>` contains elements of type
  ///     [google.cloud.resourcemanager.v3.TagHold], or rather,
  ///     the C++ class generated by Protobuf from that type. Please consult the
  ///     Protobuf documentation for details on the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.resourcemanager.v3.ListTagHoldsRequest]: @googleapis_reference_link{google/cloud/resourcemanager/v3/tag_holds.proto#L166}
  /// [google.cloud.resourcemanager.v3.TagHold]: @googleapis_reference_link{google/cloud/resourcemanager/v3/tag_holds.proto#L87}
  ///
  // clang-format on
  StreamRange<google::cloud::resourcemanager::v3::TagHold> ListTagHolds(
      std::string const& parent, Options opts = {});

  // clang-format off
  ///
  /// Lists TagHolds under a TagValue.
  ///
  /// @param request Unary RPCs, such as the one wrapped by this
  ///     function, receive a single `request` proto message which includes all
  ///     the inputs for the RPC. In this case, the proto message is a
  ///     [google.cloud.resourcemanager.v3.ListTagHoldsRequest].
  ///     Proto messages are converted to C++ classes by Protobuf, using the
  ///     [Protobuf mapping rules].
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return a [StreamRange](@ref google::cloud::StreamRange)
  ///     to iterate of the results. See the documentation of this type for
  ///     details. In brief, this class has `begin()` and `end()` member
  ///     functions returning a iterator class meeting the
  ///     [input iterator requirements]. The value type for this iterator is a
  ///     [`StatusOr`] as the iteration may fail even after some values are
  ///     retrieved successfully, for example, if there is a network disconnect.
  ///     An empty set of results does not indicate an error, it indicates
  ///     that there are no resources meeting the request criteria.
  ///     On a successful iteration the `StatusOr<T>` contains elements of type
  ///     [google.cloud.resourcemanager.v3.TagHold], or rather,
  ///     the C++ class generated by Protobuf from that type. Please consult the
  ///     Protobuf documentation for details on the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.resourcemanager.v3.ListTagHoldsRequest]: @googleapis_reference_link{google/cloud/resourcemanager/v3/tag_holds.proto#L166}
  /// [google.cloud.resourcemanager.v3.TagHold]: @googleapis_reference_link{google/cloud/resourcemanager/v3/tag_holds.proto#L87}
  ///
  // clang-format on
  StreamRange<google::cloud::resourcemanager::v3::TagHold> ListTagHolds(
      google::cloud::resourcemanager::v3::ListTagHoldsRequest request,
      Options opts = {});

 private:
  std::shared_ptr<TagHoldsConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace resourcemanager_v3
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_RESOURCEMANAGER_V3_TAG_HOLDS_CLIENT_H
